import 'package:cloud_firestore/cloud_firestore.dart';
import '../../abg_utils.dart';

Future<String?> bookingToCashMigrate(Function(double) callback) async {
  try{
    double percentage = 0;
    var querySnapshot = await FirebaseFirestore.instance.collection("booking").get();
    var oneStep = 100/querySnapshot.docs.length;

    Map<String, dynamic> _toWrite = {};

    for (var item in querySnapshot.docs) {
      if (!item.exists)
        continue;
      var _data = OrderData.fromJson(item.id, item.data());
      setDataToCalculate(_data, null);

      var order = OrderDataCache(
        id: item.id,
        time: _data.time,
        timeModify: _data.timeModify,
        status: _data.status,
        delete: _data.delete,
        customerId: _data.customerId,
        customerName: _data.customer,
        providerId: _data.providerId,
        providerName: _data.provider,
        countProducts: 1,
        subtotal: getTotal(),
        toAdmin: getTotal()*_data.taxAdmin/100,
        toDriver: 0,
        discount: getCoupon(),
        tax: getTax(),
        total: getTotal(),
        couponId: _data.couponId,
        paymentMethod: _data.paymentMethod,
        finished: _data.finished,
      );
      _toWrite.addAll({
        order.id: order.toJson(),
      });
      percentage += oneStep;
      callback(percentage);
    }
    await FirebaseFirestore.instance.collection("cache").doc("orders").set(_toWrite, SetOptions(merge:true));
  }catch(ex){
    return "bookingToCashMigrate " + ex.toString();
  }
  return null;
}

class OrderDataCache {
  String id;
  DateTime timeModify; // время воследней модификации (для listener)
  DateTime time;
  String status;
  bool delete;
  String customerId;
  String customerName; // name
  String providerId;
  List<StringData> providerName;
  //
  int countProducts;
  double subtotal;
  double discount;
  double toAdmin;
  double toDriver;
  double tax;
  double total;
  String couponId;
  //
  String paymentMethod;
  bool finished;

  OrderDataCache({
    required this.id,
    required this.time,
    required this.timeModify,
    required this.status,
    required this.delete,
    required this.customerId,
    required this.customerName,
    required this.providerId,
    required this.providerName,
    required this.countProducts,
    required this.subtotal,
    required this.discount,
    required this.toAdmin,
    required this.toDriver,
    required this.tax,
    required this.total,
    required this.couponId,
    required this.paymentMethod,
    required this.finished,
  });

  Map<String, dynamic> toJson({bool local = false}) {
    return {
      't': local ? time.toIso8601String() : DateTime.now().toUtc(),
      'q': local ? timeModify.toIso8601String() : DateTime.now().toUtc(),
      's': status,
      'd': delete,
      'c': customerId,
      'b': customerName,
      'p': providerName.map((i) => i.toJson()).toList(),
      'r': providerId,
      'a': countProducts,
      'w': subtotal,
      'e': discount,
      'm': toAdmin,
      'z': toDriver,
      'y': tax,
      'f': total,
      'g': couponId,
      'h': paymentMethod,
      'k': finished,
    };
  }

  factory OrderDataCache.fromJson(String id, Map<String, dynamic> data, {bool local = false}){
    List<StringData> _providerName = [];
    if (data['p'] != null)
      for (var element in List.from(data['p'])) {
        _providerName.add(StringData.fromJson(element));
      }
    return OrderDataCache(
      id: id,
      time: !local ? (data["t"] != null) ? data["t"].toDate() : DateTime.now()
          : DateTime.parse(data["t"]),
      timeModify: !local ? (data["q"] != null) ? data["q"].toDate() : DateTime.now()
          : DateTime.parse(data["q"]),
      providerName: _providerName,
      status: (data["s"] != null) ? data["s"] : "",
      delete: (data["d"] != null) ? data["d"] : false,
      customerId: (data["c"] != null) ? data["c"] : "",
      customerName: (data["b"] != null) ? data["b"] : "",
      providerId: (data["r"] != null) ? data["r"] : "",
      countProducts: (data["a"] != null) ? toInt(data["a"].toString()) : 0,
      subtotal: (data["w"] != null) ? toDouble(data["w"].toString()) : 0,
      discount: (data["e"] != null) ? toDouble(data["e"].toString()) : 0,
      toAdmin: (data["m"] != null) ? toDouble(data["m"].toString()) : 0,
      toDriver: (data["z"] != null) ? toDouble(data["z"].toString()) : 0,
      tax: (data["y"] != null) ? toDouble(data["y"].toString()) : 0,
      total: (data["f"] != null) ? toDouble(data["f"].toString()) : 0,
      couponId: (data["g"] != null) ? data["g"] : "",
      paymentMethod: (data["h"] != null) ? data["h"] : "",
      finished: (data["k"] != null) ? data["k"] : false,
    );
  }
}
